@@ -6,11 +6,12 @@
 A simple and efficient command-line program framework.
 """
 
-from .setting import PROG, VERSION
 from .sitter import (
-    Argument, Application, ALL, FLEX, SitterError, ParamsParseError, ArgumentError, Command, register, Options, empty,
+    Argument, App, ALL, FLEX, SitterError, ParamsParseError, ArgumentError, Command,
+    register, Options, empty,
 )
 
-__version__ = VERSION
-__name__ = PROG
+__version__ = '0.1.2'
+__name__ = 'sitter'
 __author__ = 'St·Kali <clarkmonkey@163.com>'
+__email__ = 'clarkmonkey@163.com'
